@@ -204,6 +204,7 @@ use reqwest::{
 };
 use serde::{Deserialize, Serialize};
 use serde_json::json;
+use simdutf8::basic::from_utf8;
 use url::Url;
 
 use crate::{
@@ -341,7 +342,6 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
     let mut rconfig = Config::new(&args.arg_input)
         .delimiter(args.flag_delimiter)
         .trim(csv::Trim::All)
-        .checkutf8(false)
         .no_headers(args.flag_no_headers);
 
     let mut rdr = rconfig.reader()?;
@@ -619,14 +619,14 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
             // let's dynamically construct the URL with it
             record_vec.clear();
             for field in &record {
-                record_vec.push(unsafe { std::str::from_utf8_unchecked(field).to_owned() });
+                record_vec.push(from_utf8(field).unwrap_or_default().to_owned());
             }
             if let Ok(formatted) =
                 dynfmt::SimpleCurlyFormat.format(&dynfmt_url_template, &*record_vec)
             {
                 url = formatted.into_owned();
             }
-        } else if let Ok(s) = std::str::from_utf8(&record[column_index]) {
+        } else if let Ok(s) = from_utf8(&record[column_index]) {
             // we're not using a URL template,
             // just use the field as-is as the URL
             s.clone_into(&mut url);
@@ -673,7 +673,7 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
 
                 if GET_REDIS_RESPONSE.cache_remove(&key).is_err() && log_enabled!(Warn) {
                     // failure to remove cache keys is non-fatal. Continue, but log it.
-                    warn!(r#"Cannot remove Redis key "{key}""#);
+                    wwarn!(r#"Cannot remove Redis key "{key}""#);
                 };
             }
         } else {
@@ -1139,7 +1139,7 @@ fn get_response(
             }
 
             if retries >= flag_max_retries {
-                warn!("{flag_max_retries} max-retries reached.");
+                wwarn!("{flag_max_retries} max-retries reached.");
                 break 'retry;
             }
             retries += 1;
