@@ -80,7 +80,7 @@ macro_rules! command_list {
     validate    Validate CSV data for RFC4180-compliance or with JSON Schema
 
     NOTE: qsvdp ignores the --progressbar option for all commands.
-    luau may be disabled if your platform is not compatible.
+    luau is currently disabled until Datapusher+ can take advantage of it.
 
     sponsored by datHere - Data Infrastructure Engineering
 "
@@ -269,7 +269,10 @@ impl Command {
             Command::SearchSet => cmd::searchset::run(argv),
             Command::Select => cmd::select::run(argv),
             Command::Slice => cmd::slice::run(argv),
-            Command::Sniff => cmd::sniff::run(argv),
+            Command::Sniff => {
+                let rt = tokio::runtime::Runtime::new().unwrap();
+                rt.block_on(cmd::sniff::run(argv))
+            }
             Command::Sort => cmd::sort::run(argv),
             Command::SortCheck => cmd::sortcheck::run(argv),
             Command::Stats => cmd::stats::run(argv),
