@@ -33,6 +33,8 @@ const DEFAULT_FREEMEMORY_HEADROOM_PCT: u8 = 20;
 
 static ROW_COUNT: once_cell::sync::OnceCell<u64> = OnceCell::new();
 
+pub type ByteString = Vec<u8>;
+
 #[inline]
 pub fn num_cpus() -> usize {
     num_cpus::get()
@@ -257,7 +259,7 @@ pub fn finish_progress(progress: &ProgressBar) {
             .unwrap(),
     );
 
-    if progress.length().unwrap() == progress.position() {
+    if progress.length().unwrap_or_default() == progress.position() {
         progress.finish();
         log::info!("Progress done... {}", progress.message());
     } else {
@@ -436,7 +438,7 @@ pub fn condense(val: Cow<[u8]>, n: Option<usize>) -> Cow<[u8]> {
         None => val,
         Some(n) => {
             let mut is_short_utf8 = false;
-            if let Ok(s) = str::from_utf8(&val) {
+            if let Ok(s) = simdutf8::basic::from_utf8(&val) {
                 if n >= s.chars().count() {
                     is_short_utf8 = true;
                 } else {
@@ -1045,3 +1047,21 @@ pub fn round_num(dec_f64: f64, places: u32) -> String {
         .normalize()
         .to_string()
 }
+
+#[inline]
+pub fn transform(bs: &[u8], casei: bool) -> ByteString {
+    if let Ok(s) = simdutf8::basic::from_utf8(bs) {
+        if casei {
+            let norm: String = s
+                .trim()
+                .chars()
+                .map(|c| c.to_lowercase().next().unwrap())
+                .collect();
+            norm.into_bytes()
+        } else {
+            s.trim().as_bytes().to_vec()
+        }
+    } else {
+        bs.to_vec()
+    }
+}
