@@ -43,7 +43,7 @@ fn luau_map_idx() {
     let mut cmd = wrk.command("luau");
     cmd.arg("map")
         .arg("inc")
-        .arg("number * _idx")
+        .arg("number * _IDX")
         .arg("data.csv");
 
     let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
@@ -89,7 +89,7 @@ fn luau_aggregation() {
 }
 
 #[test]
-fn luau_aggregation_with_prologue() {
+fn luau_aggregation_with_begin() {
     let wrk = Workdir::new("luau");
     wrk.create(
         "data.csv",
@@ -104,7 +104,7 @@ fn luau_aggregation_with_prologue() {
     let mut cmd = wrk.command("luau");
     cmd.arg("map")
         .arg("Total")
-        .arg("--prologue")
+        .arg("--begin")
         .arg("tot = 0")
         .arg("-x")
         .arg("tot = tot + Amount; return tot")
@@ -122,7 +122,7 @@ fn luau_aggregation_with_prologue() {
 }
 
 #[test]
-fn luau_aggregation_with_prologue_epilogue() {
+fn luau_aggregation_with_begin_end() {
     let wrk = Workdir::new("luau");
     wrk.create(
         "data.csv",
@@ -137,12 +137,12 @@ fn luau_aggregation_with_prologue_epilogue() {
     let mut cmd = wrk.command("luau");
     cmd.arg("map")
         .arg("Total")
-        .arg("--prologue")
+        .arg("--begin")
         .arg("tot = 0; gtotal = 0; amt_array = {}")
         .arg("-x")
-        .arg("amt_array[_idx] = Amount; tot = tot + Amount; gtotal = gtotal + tot; return tot")
-        .arg("--epilogue")
-        .arg(r#"return ("Min/Max: " .. math.min(unpack(amt_array)) .. "/" .. math.max(unpack(amt_array)) .. " Grand total of " .. _rowcount .. " rows: " .. gtotal)"#)
+        .arg("amt_array[_IDX] = Amount; tot = tot + Amount; gtotal = gtotal + tot; return tot")
+        .arg("--end")
+        .arg(r#"return ("Min/Max: " .. math.min(unpack(amt_array)) .. "/" .. math.max(unpack(amt_array)) .. " Grand total of " .. _ROWCOUNT .. " rows: " .. gtotal)"#)
         .arg("data.csv");
 
     let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
@@ -155,15 +155,256 @@ fn luau_aggregation_with_prologue_epilogue() {
     ];
     assert_eq!(got, expected);
 
-    let epilogue = wrk.output_stderr(&mut cmd);
-    let expected_epilogue = "Min/Max: 7/72 Grand total of 4 rows: 275\n".to_string();
-    assert_eq!(epilogue, expected_epilogue);
+    let end = wrk.output_stderr(&mut cmd);
+    let expected_end = "Min/Max: 7/72 Grand total of 4 rows: 275\n".to_string();
+    assert_eq!(end, expected_end);
 
     wrk.assert_success(&mut cmd);
 }
 
 #[test]
-fn luau_aggregation_with_prologue_epilogue_and_luau_syntax() {
+fn luau_aggregation_with_embedded_begin_end() {
+    let wrk = Workdir::new("luau_embedded_begin_end");
+    wrk.create(
+        "data.csv",
+        vec![
+            svec!["letter", "Amount"],
+            svec!["a", "13"],
+            svec!["b", "24"],
+            svec!["c", "72"],
+            svec!["d", "7"],
+        ],
+    );
+    let mut cmd = wrk.command("luau");
+    cmd.arg("map")
+        .arg("Total")
+        .arg("-x")
+        .arg(
+            "BEGIN {tot = 0; gtotal = 0; amt_array = {}}! amt_array[_IDX] = Amount; tot = tot + \
+             Amount; gtotal = gtotal + tot; return tot END {return (\"Min/Max: \" .. \
+             math.min(unpack(amt_array)) .. \"/\" .. math.max(unpack(amt_array)) .. \" Grand \
+             total of \" .. _ROWCOUNT .. \" rows: \" .. gtotal)}!",
+        )
+        .arg("data.csv");
+
+    let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
+    let expected = vec![
+        svec!["letter", "Amount", "Total"],
+        svec!["a", "13", "13"],
+        svec!["b", "24", "37"],
+        svec!["c", "72", "109"],
+        svec!["d", "7", "116"],
+    ];
+    assert_eq!(got, expected);
+
+    let end = wrk.output_stderr(&mut cmd);
+    let expected_end = "Min/Max: 7/72 Grand total of 4 rows: 275\n".to_string();
+    assert_eq!(end, expected_end);
+
+    wrk.assert_success(&mut cmd);
+}
+
+#[test]
+fn luau_aggregation_with_embedded_begin_end_using_file() {
+    let wrk = Workdir::new("luau_embedded_begin_end");
+    wrk.create(
+        "data.csv",
+        vec![
+            svec!["letter", "Amount"],
+            svec!["a", "13"],
+            svec!["b", "24"],
+            svec!["c", "72"],
+            svec!["d", "7"],
+        ],
+    );
+
+    wrk.create_from_string(
+        "testbeginend.luau",
+        r#"
+BEGIN {
+    -- this is the BEGIN block, which is executed once at the beginning
+    -- where we typically initialize variables
+    running_total = 0;
+    grand_total = 0;
+    amount_array = {};
+}!
+
+-- this is the MAIN script, which is executed for each row
+-- note how we use the _IDX special variable to get the row index
+amount_array[_IDX] = Amount;
+running_total = running_total + Amount;
+grand_total = grand_total + running_total;
+-- running_total is the value we "map" to the "Running Total" column of each row
+return running_total;
+
+END {
+    -- and this is the END block, which is executed once at the end
+    -- note how we use the _ROWCOUNT special variable to get the number of rows
+    min_amount = math.min(unpack(amount_array));
+    max_amount = math.max(unpack(amount_array));
+    return ("Min/Max: " .. min_amount .. "/" .. max_amount ..
+       " Grand total of " .. _ROWCOUNT .. " rows: " .. grand_total);
+}!        
+"#,
+    );
+
+    let mut cmd = wrk.command("luau");
+    cmd.arg("map")
+        .arg("Running Total")
+        .arg("-x")
+        .arg("file:testbeginend.luau")
+        .arg("data.csv");
+
+    let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
+    let expected = vec![
+        svec!["letter", "Amount", "Running Total"],
+        svec!["a", "13", "13"],
+        svec!["b", "24", "37"],
+        svec!["c", "72", "109"],
+        svec!["d", "7", "116"],
+    ];
+    assert_eq!(got, expected);
+
+    let end = wrk.output_stderr(&mut cmd);
+    let expected_end = "Min/Max: 7/72 Grand total of 4 rows: 275\n".to_string();
+    assert_eq!(end, expected_end);
+
+    wrk.assert_success(&mut cmd);
+}
+
+#[test]
+fn luau_aggregation_with_embedded_begin_end_and_beginend_options() {
+    // when a main script has BEGIN/END blocks, and --begin/--end options are also specified,
+    // the --begin/--end options take precedence and the embedded BEGIN/END blocks are ignored.
+    let wrk = Workdir::new("luau_embedded_begin_end_options");
+    wrk.create(
+        "data.csv",
+        vec![
+            svec!["letter", "Amount"],
+            svec!["a", "13"],
+            svec!["b", "24"],
+            svec!["c", "72"],
+            svec!["d", "7"],
+        ],
+    );
+    let mut cmd = wrk.command("luau");
+    cmd.arg("map")
+        .arg("Total")
+        .arg("--begin")
+        .arg("tot = 1; gtotal = 0; amt_array = {}")
+        .arg("-x")
+        .arg(
+            "BEGIN {tot = 0; gtotal = 0; amt_array = {}}! amt_array[_IDX] = Amount; tot = tot + \
+             Amount; gtotal = gtotal + tot; return tot END {return (\"Min/Max: \" .. \
+             math.min(unpack(amt_array)) .. \"/\" .. math.max(unpack(amt_array)) .. \" Grand \
+             total of \" .. _ROWCOUNT .. \" rows: \" .. gtotal)}!",
+        )
+        .arg("--end")
+        .arg(
+            "return (\"Minimum/Maximum: \" .. math.min(unpack(amt_array)) .. \"/\" .. \
+             math.max(unpack(amt_array)) .. \" Grand total of \" .. _ROWCOUNT .. \" rows: \" .. \
+             gtotal)",
+        )
+        .arg("data.csv");
+
+    let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
+    let expected = vec![
+        svec!["letter", "Amount", "Total"],
+        svec!["a", "13", "14"],
+        svec!["b", "24", "38"],
+        svec!["c", "72", "110"],
+        svec!["d", "7", "117"],
+    ];
+    assert_eq!(got, expected);
+
+    let end = wrk.output_stderr(&mut cmd);
+    let expected_end = "Minimum/Maximum: 7/72 Grand total of 4 rows: 279\n".to_string();
+    assert_eq!(end, expected_end);
+
+    wrk.assert_success(&mut cmd);
+}
+
+#[test]
+fn luau_aggregation_with_embedded_begin_end_using_file_random_access() {
+    let wrk = Workdir::new("luau_embedded_begin_end");
+    wrk.create_indexed(
+        "data.csv",
+        vec![
+            svec!["letter", "Amount"],
+            svec!["a", "13"],
+            svec!["b", "24"],
+            svec!["c", "72"],
+            svec!["d", "7"],
+        ],
+    );
+
+    wrk.create_from_string(
+        "testbeginend.luau",
+        r#"
+BEGIN {
+    -- this is the BEGIN block, which is executed once at the beginning
+    -- where we typically initialize variables
+    running_total = 0;
+    grand_total = 0;
+    amount_array = {};
+
+    -- start from the end of the CSV file, set _INDEX to the last row
+    -- both _IDX and _INDEX are zero-based, that's why we subtract 1
+    _INDEX = _ROWCOUNT - 1;
+}!
+
+-- this is the MAIN script, which is executed for each row
+-- note how we use the _IDX to get the row index
+amount_array[_IDX] = Amount;
+running_total = running_total + Amount;
+grand_total = grand_total + running_total;
+
+-- note how we use the qsv_log function to log to the qsv log file
+qsv_log("warn", "logging from Luau script! running_total:", running_total, " _INDEX:", _INDEX)
+
+-- we modify _INDEX to do random access on the CSV file, in this case going backwards
+_INDEX = _INDEX - 1;
+
+-- running_total is the value we "map" to the "Running Total" column of each row
+return running_total;
+
+END {
+    -- and this is the END block, which is executed once at the end
+    -- note how we use the _ROWCOUNT special variable to get the number of rows
+    min_amount = math.min(unpack(amount_array));
+    max_amount = math.max(unpack(amount_array));
+    return ("Min/Max: " .. min_amount .. "/" .. max_amount ..
+       " Grand total of " .. _ROWCOUNT .. " rows: " .. grand_total);
+}!
+"#,
+    );
+
+    let mut cmd = wrk.command("luau");
+    cmd.arg("map")
+        .arg("Running Total")
+        .arg("-x")
+        .arg("file:testbeginend.luau")
+        .arg("data.csv");
+
+    let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
+    let expected = vec![
+        svec!["letter", "Amount", "Running Total"],
+        svec!["d", "7", "7"],
+        svec!["c", "72", "79"],
+        svec!["b", "24", "103"],
+        svec!["a", "13", "116"],
+    ];
+    assert_eq!(got, expected);
+
+    let end = wrk.output_stderr(&mut cmd);
+    let expected_end = "Min/Max: 7/72 Grand total of 4 rows: 305\n".to_string();
+    assert_eq!(end, expected_end);
+
+    wrk.assert_success(&mut cmd);
+}
+
+#[test]
+fn luau_aggregation_with_begin_end_and_luau_syntax() {
     let wrk = Workdir::new("luau");
     wrk.create(
         "data.csv",
@@ -178,12 +419,12 @@ fn luau_aggregation_with_prologue_epilogue_and_luau_syntax() {
     let mut cmd = wrk.command("luau");
     cmd.arg("map")
         .arg("Total")
-        .arg("--prologue")
+        .arg("--begin")
         .arg("tot = 0; gtotal = 0; amt_array = {}")
         .arg("-x")
-        .arg("amt_array[_idx] = Amount; tot += if tonumber(Amount) < 0 then Amount * -1 else Amount; gtotal += tot; return tot")
-        .arg("--epilogue")
-        .arg(r#"return ("Min/Max: " .. math.min(unpack(amt_array)) .. "/" .. math.max(unpack(amt_array)) .. " Grand total of " .. _rowcount .. " rows: " .. gtotal)"#)
+        .arg("amt_array[_IDX] = Amount; tot += if tonumber(Amount) < 0 then Amount * -1 else Amount; gtotal += tot; return tot")
+        .arg("--end")
+        .arg(r#"return ("Min/Max: " .. math.min(unpack(amt_array)) .. "/" .. math.max(unpack(amt_array)) .. " Grand total of " .. _ROWCOUNT .. " rows: " .. gtotal)"#)
         .arg("data.csv");
 
     let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
@@ -196,9 +437,9 @@ fn luau_aggregation_with_prologue_epilogue_and_luau_syntax() {
     ];
     assert_eq!(got, expected);
 
-    let epilogue = wrk.output_stderr(&mut cmd);
-    let expected_epilogue = "Min/Max: -72/13 Grand total of 4 rows: 275\n".to_string();
-    assert_eq!(epilogue, expected_epilogue);
+    let end = wrk.output_stderr(&mut cmd);
+    let expected_end = "Min/Max: -72/13 Grand total of 4 rows: 275\n".to_string();
+    assert_eq!(end, expected_end);
 
     wrk.assert_success(&mut cmd);
 }
